@@ -44,6 +44,8 @@ using exactextract::RasterSource;
 
 // [[Rcpp::export]]
 Rcpp::List CPP_exact_extract(Rcpp::S4 & rast,
+                             const Rcpp::NumericVector & rast_ext,
+                             const Rcpp::NumericVector & rast_res,
                              Rcpp::Nullable<Rcpp::S4> & rast_uncropped,
                              Rcpp::Nullable<Rcpp::S4> & weights,
                              const Rcpp::RawVector & wkb,
@@ -68,7 +70,7 @@ Rcpp::List CPP_exact_extract(Rcpp::S4 & rast,
     auto weights_grid = exactextract::Grid<bounded_extent>::make_empty();
     auto common_grid = grid;
 
-    S4RasterSource rsrc(rast, default_value);
+    S4RasterSource rsrc(rast, rast_ext, rast_res, default_value);
     int src_nlayers = get_nlayers(rast);
 
     std::unique_ptr<S4RasterSource> rweights;
@@ -240,9 +242,30 @@ enum class WeightingMethod {
   AREA
 };
 
+
+static int get_num_stats(const Rcpp::StringVector & stats,
+                  const std::size_t num_quantiles,
+                  const std::size_t num_unique_values) {
+  int num_stats = 0;
+  for (const auto & stat : stats) {
+      if (stat == std::string("frac") ||
+          stat == std::string("weighted_frac")) {
+        num_stats += static_cast<int>(num_unique_values);
+      } else if (stat == std::string("quantile")) {
+        num_stats += static_cast<int>(num_quantiles);
+      } else {
+        num_stats += 1;
+      }
+  }
+
+  return num_stats;
+}
+
 // Return a matrix with one row per stat and one row per raster layer
 // [[Rcpp::export]]
 Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
+                              const Rcpp::NumericVector & rast_ext,
+                              const Rcpp::NumericVector & rast_res,
                               Rcpp::Nullable<Rcpp::S4> weights,
                               const Rcpp::RawVector & wkb,
                               double default_value,
@@ -262,7 +285,7 @@ Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
 
     int nlayers = get_nlayers(rast);
 
-    S4RasterSource rsrc(rast, default_value);
+    S4RasterSource rsrc(rast, rast_ext, rast_res, default_value);
 
     std::unique_ptr<S4RasterSource> rweights;
     std::string area_method;
@@ -296,7 +319,10 @@ Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
     bool disaggregated = (grid.dx() < rsrc.grid().dx() || grid.dy() < rsrc.grid().dy());
 
     bool store_values = false;
-    int stat_result_size = 0;
+    bool calc_value_set = false;
+    std::size_t num_unique_values = 0;
+    std::size_t num_quantiles = 0;
+
     for (const auto & stat : stats) {
       store_values = store_values || requires_stored_values(stat);
 
@@ -305,9 +331,12 @@ Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
         Rcpp::stop("Cannot compute 'count' or 'sum' when value raster is disaggregated to resolution of weights.");
       }
 
-      if (stat == std::string("quantile")) {
-        int num_quantiles = 0;
+      if (stat == std::string("frac") ||
+          stat == std::string("weighted_frac")) {
+        calc_value_set = true;
+      }
 
+      if (stat == std::string("quantile")) {
         if (quantiles.isNotNull()) {
           Rcpp::NumericVector qvec = quantiles.get();
           num_quantiles = qvec.size();
@@ -316,10 +345,6 @@ Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
         if (num_quantiles == 0) {
           Rcpp::stop("Quantiles not specified.");
         }
-
-        stat_result_size += num_quantiles;
-      } else {
-        stat_result_size += 1;
       }
     }
 
@@ -331,8 +356,6 @@ Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
       raster_stats.emplace_back(store_values);
     }
 
-    Rcpp::NumericMatrix stat_results = Rcpp::no_init(nresults, stat_result_size);
-
     if (bbox.intersects(grid.extent())) {
       auto cropped_grid = grid.crop(bbox);
 
@@ -393,6 +416,27 @@ Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
       }
     }
 
+
+    std::set<double> value_set;
+    if (calc_value_set) {
+      for (const auto& rs : raster_stats) {
+        for (const auto& value : rs) {
+          value_set.insert(value);
+        }
+      }
+      num_unique_values = value_set.size();
+    }
+
+    auto stat_result_size = get_num_stats(stats, num_quantiles, num_unique_values);
+    Rcpp::NumericMatrix stat_results = Rcpp::no_init(nresults, stat_result_size);
+
+    if (calc_value_set) {
+      Rcpp::NumericVector unique_values(value_set.begin(), value_set.end());
+      stat_results.attr("unique_values") = unique_values;
+    }
+
+    // rows (j) represent layers
+    // cols (i) represent stats
     for (int j = 0; j < nresults; j++) {
       const auto& rs = raster_stats[j];
 
@@ -427,10 +471,24 @@ Rcpp::NumericMatrix CPP_stats(Rcpp::S4 & rast,
           }
         }
 
+        else if (stat == std::string("frac")) {
+          for (double v : value_set) {
+            stat_results(j, i++) = rs.frac(v).value_or(0);
+          }
+        }
+
+        else if (stat == std::string("weighted_frac")) {
+          for (double v : value_set) {
+            stat_results(j, i++) = rs.weighted_frac(v).value_or(0);
+          }
+        }
+
         else Rcpp::stop("Unknown stat: " + stat);
       }
     }
 
+    // FIXME set dimnames here
+
     return stat_results;
   } catch (std::exception & e) {
     // throw predictable exception class
